/*
   fenris - program execution path analysis tool
   ---------------------------------------------

   Copyright (C) 2001, 2002 by Bindview Corporation
   Portions copyright (C) 2001, 2002 by their respective contributors
   Developed and maintained by Michal Zalewski <lcamtuf@coredump.cx>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   This code is taken from cxmon utility, http://www.uni-mainz.de/~bauec002/,
   written by Christian Bauer and Marc Hellwig. Most of their code was
   taken from GNU binutils libopcode for i386, which is probably
   authored mostly by Pace Willisson. I made several changes, changed
   the notation a bit and removed some code, but I'm not the one to take
   credit for the functionality of this code (or lack of thereof ;-).

*/

#include <stdarg.h>
#include <ncurses.h>
#include "dis-asm.h"

unsigned int base_addr;
char* global_buffer;


char mon_read_byte(int where) { return global_buffer[where-base_addr]; }


int buffer_read_memory(bfd_vma from, bfd_byte *to, int length, struct disassemble_info *info) {
  while (length--) *to++ = mon_read_byte(from++);
  return 0;
}


void perror_memory(int status, bfd_vma memaddr, struct disassemble_info *info) {
  info->fprintf_func(info->stream, "Unknown error %d\n", status);
}


extern char* describe_address(unsigned int addr);


void generic_print_address(bfd_vma addr, struct disassemble_info *info) {
  char* x;
  info->fprintf_func(info->stream,"$0x%x",addr);
  x=describe_address(addr);
  if (strlen(x)) info->fprintf_func(info->stream," <%s>",x);
}


int generic_symbol_at_address(bfd_vma addr, struct disassemble_info *info) {
  return 0;
}


struct SFILE { char *buffer; char *current; };


static int mon_sprintf(struct SFILE *f, const char *format, ...) {
  int n;
  va_list args;
  va_start(args, format);
  vsprintf(f->current, format, args);
  f->current += n = strlen(f->current);
  va_end(args);
  return n;
}


int disass_x86(FILE *f, unsigned int adr) {
  char* x;
  disassemble_info info;
  int num,i;
  char buf[1024];
  struct SFILE sfile = {buf, buf};
  sfile.buffer = buf;
  sfile.current = buf;
  INIT_DISASSEMBLE_INFO(info, (FILE *)&sfile, (fprintf_ftype)mon_sprintf);
  num = print_insn_i386(adr, &info);
  fprintf(f,"%08x",adr);
  x=describe_address(adr);
  if (strlen(x)) fprintf(f," [%s]",x);
  fprintf(f, ": \t%s\n", buf);
  return num;
}


void disass (const char* buf,unsigned int addr,unsigned int len) {
  int add=addr;
  base_addr=addr;
  global_buffer=(char*)buf;

  while (add <= addr+len) 
   add+=disass_x86(stdout,add);

}

int wdisass_x86(WINDOW * w, unsigned int adr) {
  char* x;
  disassemble_info info;
  int num,i;
  char buf[1024];
  struct SFILE sfile = {buf, buf};
  sfile.buffer = buf;
  sfile.current = buf;
  INIT_DISASSEMBLE_INFO(info, (FILE *)&sfile, (fprintf_ftype)mon_sprintf);
  num = print_insn_i386(adr, &info);
  wprintw(w,"%08x",adr);
  x=describe_address(adr);
  if (strlen(x)) wprintw(w," [%s]",x);
  wprintw(w, ": \t%s\n", buf);
  return num;
}


void wdisass (const char* buf,unsigned int addr,unsigned int len,WINDOW *w) {
  int add=addr;
  base_addr=addr;
  global_buffer=(char*)buf;

  while (add <= addr+len){
   add+=wdisass_x86(w,add);
  }
}

int w1disass (const char* buf, unsigned int addr, WINDOW * w){
base_addr=addr;
global_buffer=(char*)buf;
return wdisass_x86(w,addr);
}

int getopsize (const char* buf, unsigned int addr){
  char* x;
  disassemble_info info;
  int num,i;
  char buf2[1024];
  struct SFILE sfile = {buf2, buf2};
  base_addr=addr;
  global_buffer=(char*)buf;
  sfile.buffer = buf2;
  sfile.current = buf2;
  INIT_DISASSEMBLE_INFO(info, (FILE *)&sfile, (fprintf_ftype)mon_sprintf);
  num = print_insn_i386(addr, &info);
  return num;
}
